@@ -4,6 +4,7 @@ import shutil
 import numpy as np
 
 from guide_bot.parameters import instrument_parameters as ipars
+from guide_bot.base_elements.base_element_geometry import GuidePoint
 
 
 def handle_input_parameter(name, par, default_min=None, default_max=None):
@@ -128,7 +129,7 @@ class Element:
         if isinstance(start_point, ipars.InstrumentParameter):
             raise ValueError("start_point parameter not allowed to be custom parameter.")
 
-        self.length = handle_input_parameter(name + "_length", length)
+        self.length = handle_input_parameter(name + "_length", length, default_min=0.1)
         self.start_point = handle_input_parameter(name + "_start_point", start_point)
         
         self.start_point_parameter = None
@@ -142,6 +143,9 @@ class Element:
 
         self.owner = None
 
+    def get_name(self):
+        return self.name
+
     def get_length_name(self):
         """
         Provides length name, ensures this variable is available in instrument
@@ -276,8 +280,6 @@ class GuideElement(Element):
         self.current_instrument = None
         self.current_parameters = None
 
-
-
         super().__init__(name, length=length, start_point=start_point, **kwargs)
     
     def setup_instrument_and_parameters(self, instrument, instrument_parameters):
@@ -293,6 +295,36 @@ class GuideElement(Element):
         self.current_parameters.add_parameter(self.start_width)
         self.current_parameters.add_parameter(self.start_height)
 
+    def connect_to_new_parameters(self, instrument_parameters):
+        """
+        Generic method to connect all parameter attributes to new parameter container
+
+        When switching parameter container for the element, the connections from the
+        parameter attributes to the container parameters are lost, and they need to be
+        reconnected. This method finds all instances of InstrumentParameter attributes
+        on the element object and connects them with versions that have an identical
+        name in the new instrument_parameters container.
+        """
+
+        # Find all instrument parameter objects in current parameters
+        attributes_to_instr_par_names = {}
+        attributes = vars(self)
+        for attr_name, attr in attributes.items():
+            if isinstance(attr, ipars.InstrumentParameter):
+                attributes_to_instr_par_names[attr_name] = attr.name
+
+        # Switch to new instrument parameters container
+        self.current_parameters = instrument_parameters
+
+        # Link the parameters to the ones with same name in new instrument_parameters
+        parameter_names = [x.name for x in self.current_parameters.all_parameters]
+        for attr_name in attributes_to_instr_par_names:
+            instr_par_name = attributes_to_instr_par_names[attr_name]
+            if instr_par_name in parameter_names:
+                par_index = parameter_names.index(instr_par_name)
+                print("reconnecting: ", attr_name, " to ", instr_par_name)
+                setattr(self, attr_name, self.current_parameters.all_parameters[par_index])
+
     def create_new_parameter(self, *args, **kwargs):
         """
         Uncertain about the need for this method
@@ -338,38 +370,6 @@ class GuideElement(Element):
         self.end_width = end_width
         self.end_height = end_height
 
-    def write_to_log_base(self, file):
-        """
-        Base method for writing basic information to guide log file, not to be overwritten
-        """
-
-        file.write("\nElement " + self.__class__.__name__ + " " + self.name + "\n")
-        self.write_parameter(file, "start_point", self.start_point_parameter, "start_point")
-        self.write_parameter(file, "next_start_point", self.next_start_point_parameter, "start_point")
-        self.write_parameter(file, "start_width", self.start_width, "horizontal")
-        self.write_parameter(file, "start_height", self.start_height, "vertical")
-        self.write_parameter(file, "end_width", self.end_width, "horizontal")
-        self.write_parameter(file, "end_height", self.end_height, "vertical")
-
-    def write_parameter(self, file, log_name, parameter, type_string=None):
-        """
-        Method for writing to log file in order to keep consistency and do error checks
-        """
-        if type_string not in ("horizontal", "vertical", "start_point", None):
-            raise RuntimeError("type_string needs to be horizontal, vertical, start_point or None")
-
-        if type_string is None:
-            type_string = ""
-
-        file.write(" " + log_name.ljust(29) + type_string.ljust(15) + parameter.name.ljust(50) + "\n")
-
-    def write_to_log(self, file):
-        """
-        Template function which should be overwritten for guide elements to add more details as necessary
-        """
-
-        self.write_to_log_base(file)
-
     def copy_component(self, name, destination):
         this_file_path = os.path.abspath(__file__)
         this_directory = os.path.split(this_file_path)[0]
@@ -381,6 +381,73 @@ class GuideElement(Element):
     def copy_components(self, destination):
         return []
 
+    def get_geometry(self):
+        """
+        Method to be overwritten by each element, returns geometry object
+        """
+        raise RuntimeError("Base method that must be overwritten.")
+
+    def write_to_log(self, file):
+        """
+        Uses geometry object to write log file
+        """
+
+        geometry = self.get_geometry()
+        geometry.write_to_log(file, self.__class__.__name__, self.name)
+
+    def get_los_points(self, start_pr, distance_unit_less=None, los_start=None, los_end=None):
+        """
+        Provides GuidePoints in this guide section
+
+        Sets a point for distance_unit_less, los_start and los_end. The element
+        may add extra points using the extra_los_goals method.
+
+        Parameters
+        ----------
+        start_pr : PositionAndRotation
+            Position/Rotation at the start of the element
+
+        Keyword Parameters
+        ------------------
+
+        distance_unit_less : float between 0 and 1
+            How far along this guide element the position should be
+
+        los_start : ElementPoint
+            An element_point object describing where in the element the los_start is
+
+        los_end : ElementPoint
+            An element_point object describing where in the element the los_end is
+        """
+
+        guide_points = []
+
+        if distance_unit_less is not None:
+            guide_points.append(GuidePoint(self, start_pr, distance_unit_less=distance_unit_less))
+
+        if los_start is not None:
+            guide_points.append(GuidePoint(self, start_pr, element_point=los_start))
+
+        if los_end is not None:
+            guide_points.append(GuidePoint(self, start_pr, element_point=los_end))
+
+        guide_points += self.extra_los_points(start_pr=start_pr, distance_unit_less=distance_unit_less,
+                                              los_start=los_start, los_end=los_end)
+
+        return guide_points
+
+    def extra_los_points(self, start_pr, distance_unit_less=None, los_start=None, los_end=None):
+        """
+        Method to add extra line of sight goals, to be overwritten
+        """
+        return []
+
+    def extra_repr(self):
+        """
+        Called by __repr__ and can be overwritten to write extra information
+        """
+        return None
+
     def __repr__(self):
         """
         Provides a basic description of the guide element
@@ -406,55 +473,87 @@ class GuideElement(Element):
         string += "start par:".ljust(name_length) + self.start_point_parameter.__repr__() + "\n"
         string += "n start p:".ljust(name_length) + self.next_start_point_parameter.__repr__()
 
+        extra_string = self.extra_repr()
+        if extra_string is not None:
+            string += "\n" + extra_string
+
         return string
 
 
-class PositionAndRotation:
-    def __init__(self, position, rotation):
-        self.position = position
-        self.rotation = rotation
+class LosBreakerGuideElement(GuideElement):
+    def __init__(self, name, length=None, start_point=None, start_width=None, start_height=None,
+                 end_width=None, end_height=None, **kwargs):
 
-    def width_points(self, width):
+        super().__init__(name, length=length, start_point=start_point,
+                         start_width=start_width, start_height=start_height,
+                         end_width=end_width, end_height=end_height,
+                         **kwargs)
 
-        x_vector = np.array([0.5, 0, 0])
-        x_vector = self.rotation.apply(x_vector)
+        self.dynamic_los_breaker = True
+        self.last_los_value = None
 
-        m_position = self.position - width * x_vector
-        p_position = self.position + width * x_vector
-
-        return m_position, p_position
-
-    def height_points(self, height):
-
-        y_vector = np.array([0, 0.5, 0])
-        y_vector = self.rotation.apply(y_vector)
-
-        m_position = self.position - height * y_vector
-        p_position = self.position + height * y_vector
-
-        return m_position, p_position
-
-    def get_points(self, distance, horizontal):
         """
-        Provides 3D points for plotting
-
-        Takes distance between points and wether it is in the
-        horizontal direction or vertical (True / False).
-
-        distance: float
-            distance between points
-
-        horizontal: bool
-            are they spaced horizontally or vertically
+        How to handle sign ? +/- for left right is common
+        
+        Could add a sign to LosInstrumentParameter that is applied at get_value
+        
+        Need to have enable / disable switch.
+        Cases:
+        kink at moderator can be for performance reasons
+        kink can be static
+        kink can be meant to block line of sight
+        
+        If user sets angle, it is static. Disable this as a los_blocker
+        If user sets nothing, what to do then? Break los if there is a section, error otherwise?
+        If for performance reasons, user could set a optimize this please flag
         """
-        if horizontal:
-            return self.width_points(distance)
-        else:
-            return self.height_points(distance)
 
-    def __repr__(self):
-        string = "PositionAndRotation\n"
-        string += " Position: " + str(self.position) + "\n"
-        string += " Rotation: \n" + str(self.rotation.as_matrix()) + "\n"
+    def set_los_value(self, value):
+        """
+        Saves los value and transfers to overwritten los_input method
 
-        return string
\ No newline at end of file
+        Parameters
+        ----------
+        value : float
+            Control value for this line of sight breaker
+        """
+
+        self.last_los_value = value
+        self.los_input(value)
+
+    def get_los_value(self):
+        """
+        Get the los value that would recreate the current los state
+
+        The los_input method uses a positive float value to set a more complex
+        los state of the los breaker. Get los value retrieves the last los
+        value given to an element, meaning the output of this method has to be
+        zero or a positive float number. It should roughly correspond to the
+        angle deviation the los breaker introduces.
+
+        Returns
+        -------
+        los_value corresponding to state of element
+        """
+        return self.last_los_value
+
+    def los_input(self, value):
+        """
+        Input for breaking line of sight, single value input
+
+        The los input is a single value with the property that when it is
+        increased, the los breaker becomes more likely to break line of sight.
+        At a value of 0.0, it is assumed it wont break line of sight.
+        This could be the angle in a kink, or the angle a curved guide turns,
+        but it is also allowed to have several parameters controlled by the
+        los_input. It is also possible to for example invert the sign, so that
+        a kink could change directions depending on some other parameter.
+
+        Parameters
+        ----------
+
+        value : float
+            Control value for this line of sight breaker
+        """
+
+        raise RuntimeError("los_input must be overwritten by LosBreaker element.")
\ No newline at end of file
