@@ -3,11 +3,13 @@ import numpy as np
 from scipy.spatial.transform import Rotation as R
 
 from guide_bot.base_elements import guide_elements
-from guide_bot.base_elements.guide_elements import PositionAndRotation
+from guide_bot.base_elements.base_element_geometry import BaseElementGeometry
+from guide_bot.base_elements.base_element_geometry import PositionAndRotation
 from guide_bot.parameters import instrument_parameters as ipars
+from guide_bot.base_elements.base_element_geometry import GuidePoint
 
 
-class Curved(guide_elements.GuideElement):
+class Curved(guide_elements.LosBreakerGuideElement):
     """
     Curved GuideElement that inserts a curved section into a guide
 
@@ -52,10 +54,10 @@ class Curved(guide_elements.GuideElement):
 
         super().__init__(name, length=length, start_point=start_point,
                          start_width=start_width, start_height=start_height,
-                         end_width=None, end_height=None,
                          **kwargs)
 
         # This McStas component only supports constant cross section
+        # start_width and start_height becomes properties that update end_width and end_height
         self.end_width = self.start_width
         self.end_height = self.start_height
 
@@ -88,18 +90,21 @@ class Curved(guide_elements.GuideElement):
 
         # How to handle a los parameter?
         if angle is None:
-            # automatic los, not made yet
-            raise ValueError("Need to specify angular_diversion")
+            # automatic los
+            self.dynamic_los_breaker = True
+            self.angular_diversion = ipars.LosInstrumentParameter(self.name + "_angular_diversion")
         else:
             if isinstance(angle, ipars.FreeInstrumentParameter):
                 raise ValueError("Not supported to have angular_diversion as a free parameter")
             elif isinstance(angle, ipars.InstrumentParameter):
                 # Fixed or dependent
+                self.dynamic_los_breaker = False
                 self.angular_diversion = angle
             elif isinstance(angle, (float, int)):
+                self.dynamic_los_breaker = False
                 self.angular_diversion = ipars.FixedInstrumentParameter(self.name + "_angular_diversion", angle)
             else:
-                raise ValueError("Unknown type of angular_diversion")
+                raise ValueError("Unknown type of angle")
 
         # temporary reflectivity model
         if "R0" in kwargs:
@@ -127,6 +132,36 @@ class Curved(guide_elements.GuideElement):
         else:
             self.W = 0.003
 
+    @property
+    def start_width(self):
+        """
+        Make start_width property so end_width can be updated with it
+        """
+        return self._start_width
+
+    @start_width.setter
+    def start_width(self, value):
+        """
+        Updates end_width to keep it in line with start_width
+        """
+        self.end_width = value
+        self._start_width = value
+
+    @property
+    def start_height(self):
+        """
+        Make start_height property so end_height can be updated with it
+        """
+        return self._start_height
+
+    @start_height.setter
+    def start_height(self, value):
+        """
+        Updates end_height to keep it in line with start_height
+        """
+        self.end_height = value
+        self._start_height = value
+
     def copy_components(self, destination):
         components_to_copy = ["Guide_curved_gravity_safe.comp"]
 
@@ -134,6 +169,97 @@ class Curved(guide_elements.GuideElement):
 
         return components_to_copy
 
+    def los_input(self, value):
+        """
+        Input for breaking line of sight, single value input
+
+        The los input is a single value with the property that when it is
+        increased, the los breaker becomes more likely to break line of sight.
+        At a value of 0.0, it is assumed it wont break line of sight.
+        If a value of 10 is reached and line of sight is not broken, an error
+        is raised by the line of sight solver.
+        This could be the angle in a kink, or the angle a curved guide turns,
+        but it is also allowed to have several parameters controlled by the
+        los_input. It is also possible to for example invert the sign, so that
+        a kink could change directions depending on some other parameter.
+
+        For kink the los input controls the kink angle, starting at 0.0. The
+        user input is used to determine the sign.
+
+        Parameters
+        ----------
+
+        value : float
+            Control value for this line of sight breaker
+        """
+        #if not isinstance(self.angular_diversion, ipars.LosInstrumentParameter):
+        #    return
+
+        # currently in radians, needs to be changed
+        radian_value = value * np.pi / 180.0
+        self.angular_diversion.set_value(radian_value)  # direction handled elsewhere
+
+    def get_los_status(self):
+        """
+        Get the los value that would recreate the current los state
+
+        The los_input method uses a positive float value to set a more complex
+        los state of the los breaker. Get los value retrieves such a value
+        from the los element, meaning the output of this method has to be
+        zero or a positive float number. It should roughly correspond to the
+        angle deviation the los breaker introduces.
+
+        Returns
+        -------
+
+        los_value corresponding to state of element
+        """
+
+        radian_value = self.angular_diversion.get_value()
+        degree_value = radian_value * 180 / np.pi
+        return abs(degree_value)
+
+    def extra_los_points(self, start_pr, distance_unit_less=None, los_start=None, los_end=None):
+        """
+        Curved guide needs additional los_points and goal at the middle.
+        """
+
+        has_start = los_start is not None
+        has_end = los_end is not None
+
+        start = self.start_point_parameter.get_value()
+        end = self.next_start_point_parameter.get_value()
+        length = end - start
+
+        if not has_start and not has_end:
+            # Neither los start or end in the segment, set distance unit less to 0.5
+            mid_frac = 0.5
+        elif has_start and not has_end:
+            # Set an extra point midway between start point and end of segment
+            start_frac = los_start.get_fraction(length)
+            mid_frac = 0.5*(start_frac + 1.0)
+        elif not has_start and has_end:
+            # Set an extra point midway between start of segment and end point
+            end_frac = los_end.get_fraction(length)
+            mid_frac = 0.5 * end_frac
+        elif has_start and has_end:
+            # Set an extra point midway between start point and end point
+            start_frac = los_start.get_fraction(length)
+            end_frac = los_end.get_fraction(length)
+            mid_frac = 0.5*(start_frac + end_frac)
+        else:
+            raise RuntimeError("Error in curved element extra los_points.")
+
+        return [GuidePoint(self, start_pr=start_pr, distance_unit_less=mid_frac)]
+
+    def extra_repr(self):
+        name_length = 25
+        string = ""
+        string += "angular_diversion:".ljust(name_length) + self.angular_diversion.__repr__() + "\n"
+        string += "horizontal_bend_par:".ljust(name_length) + self.horizontal_bend_par.__repr__() + "\n"
+        string += "positive_bend_par:".ljust(name_length) + self.positive_bend_par.__repr__() + "\n"
+        return string
+
     def add_to_instr(self):
         """
         Adds code describing the straight element to the current McStas instrument
@@ -230,93 +356,75 @@ class Curved(guide_elements.GuideElement):
         end_arm.set_AT([end_x_par_name, end_y_par_name, end_z_par_name], RELATIVE=self.reference_component_name)
         end_arm.set_ROTATED(end_rot, RELATIVE=self.reference_component_name)
 
-    def write_to_log(self, file):
-        self.write_to_log_base(file)
-        if self.horizontal_bend:
+    def get_geometry(self):
+        return GeometryCurved(start_point=self.start_point_parameter, next_start_point=self.next_start_point_parameter,
+                              start_width=self.start_width, start_height=self.start_height,
+                              end_width=self.end_width, end_height=self.end_height,
+                              angular_diversion=self.angular_diversion,
+                              bend_horizontal=self.horizontal_bend_par, positive_bend_par=self.positive_bend_par)
+
+
+class GeometryCurved(BaseElementGeometry):
+    def __init__(self, start_point, next_start_point, start_width, start_height, end_width, end_height,
+                 angular_diversion, bend_horizontal, positive_bend_par):
+
+        super().__init__(start_point=start_point, next_start_point=next_start_point,
+                         start_width=start_width, start_height=start_height,
+                         end_width=end_width, end_height=end_height)
+
+        if self.evaluate(bend_horizontal) == 1:
             direction = "horizontal"
         else:
             direction = "vertical"
 
-        self.write_parameter(file, "angular_diversion", self.angular_diversion, direction)
-        self.write_parameter(file, "bend_horizontal", self.horizontal_bend_par, direction)
-        self.write_parameter(file, "bend_direction", self.positive_bend_par, direction)
+        self.set_parameter_attribute("angular_diversion", angular_diversion, par_type=direction)
+        self.set_parameter_attribute("bend_horizontal", bend_horizontal, par_type=direction)
+        self.set_parameter_attribute("positive_bend_par", positive_bend_par, par_type=direction)
 
+        self.plot_resolution = 60  # Number of points to plot
 
-def plot_element_horizontal(ax, par_dict, color):
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_width = par_dict["start_width"]
-    end_width = par_dict["end_width"]
+    def continue_center_line(self, start_pr, distance_unit_less):
+        length = self.next_start_point - self.start_point
 
-    ax.plot([start, end], [0.5 * start_width, 0.5 * end_width], color=color)
-    ax.plot([start, end], [-0.5 * start_width, -0.5 * end_width], color=color)
+        if self.angular_diversion == 0.0:
+            # Handle no angular_deviation by setting large curvature
+            curvature = 1E9
+            rotation = 0.0
+            displacement_z = length*distance_unit_less
+        else:
+            curvature = length / self.angular_diversion
+            rotation = distance_unit_less * self.angular_diversion
+            displacement_z = curvature * np.sin(rotation)
 
+        displacement = self.positive_bend_par * curvature * (1 - np.cos(rotation))
+        if self.bend_horizontal == 1:
+            displacement_x = displacement
+            displacement_y = 0
+        elif self.bend_horizontal == -1:
+            displacement_x = 0
+            displacement_y = displacement
+        else:
+            raise RuntimeError("Parameter bend_horizontal must be either 1 or -1.")
 
-def plot_element_vertical(ax, par_dict, color):
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_height = par_dict["start_height"]
-    end_height = par_dict["end_height"]
+        x_vector = np.array([1, 0, 0])
+        y_vector = np.array([0, 1, 0])
+        z_vector = np.array([0, 0, 1])
 
-    ax.plot([start, end], [0.5 * start_height, 0.5 * end_height], color=color)
-    ax.plot([start, end], [-0.5 * start_height, -0.5 * end_height], color=color)
+        start_direction = start_pr.rotation
 
+        x_vector = start_direction.apply(x_vector)
+        y_vector = start_direction.apply(y_vector)
+        z_vector = start_direction.apply(z_vector)
 
-def guide_width(par_dict, distance_unit_less):
-    return par_dict["start_width"]
+        updated_position = start_pr.position + displacement_x*x_vector + displacement_y*y_vector + displacement_z*z_vector
+        if self.bend_horizontal == 1:
+            updated_rotation = start_pr.rotation * R.from_euler("y", self.positive_bend_par * rotation)
+        elif self.bend_horizontal == -1:
+            updated_rotation = start_pr.rotation * R.from_euler("x", -self.positive_bend_par * rotation)
+        else:
+            raise RuntimeError("Parameter bend_horizontal must be either 1 or -1.")
 
-def guide_height(par_dict, distance_unit_less):
-    return par_dict["start_height"]
-
-def guide_dimensions(par_dict, distance_unitless, horizontal):
-    if horizontal:
-        return par_dict["start_width"]
-    else:
-        return par_dict["start_height"]
-
-def center_line(start_pr, par_dict, distance_unit_less):
-
-    angular_diversion = par_dict["angular_diversion"]
-    bend_horizontal = par_dict["bend_horizontal"]
-    bend_direction = par_dict["bend_direction"]
-
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
-    start_height = par_dict["start_height"]
-    end_height = par_dict["end_height"]
-
-    length = end - start
-    curvature = length/angular_diversion
-
-    rotation = distance_unit_less*angular_diversion
-
-    displacement = bend_direction*curvature*(1-np.cos(rotation))
-    if bend_horizontal == 1:
-        displacement_x = displacement
-        displacement_y = 0
-    elif bend_horizontal == -1:
-        displacement_x = 0
-        displacement_y = displacement
-
-    displacement_z = curvature*np.sin(rotation)
-
-    x_vector = np.array([1, 0, 0])
-    y_vector = np.array([0, 1, 0])
-    z_vector = np.array([0, 0, 1])
-
-    start_direction = start_pr.rotation
-
-    x_vector = start_direction.apply(x_vector)
-    y_vector = start_direction.apply(y_vector)
-    z_vector = start_direction.apply(z_vector)
-
-    updated_position = start_pr.position + displacement_x*x_vector + displacement_y*y_vector + displacement_z*z_vector
-    if bend_horizontal == 1:
-        updated_rotation = start_pr.rotation * R.from_euler("y", bend_direction * rotation)
-    elif bend_horizontal == -1:
-        updated_rotation = start_pr.rotation * R.from_euler("x", -bend_direction * rotation)
-
-    return PositionAndRotation(updated_position, updated_rotation)
+        return PositionAndRotation(updated_position, updated_rotation)
 
 
 
