@@ -1,5 +1,8 @@
 from guide_bot.base_elements import guide_elements
 
+from guide_bot.base_elements.base_element_geometry import BaseElementGeometry
+from guide_bot.base_elements.base_element_geometry import PositionAndRotation
+
 class Slit(guide_elements.GuideElement):
     """
     Gap GuideElement that inserts an empty space into a guide
@@ -76,32 +79,42 @@ class Slit(guide_elements.GuideElement):
         end = self.current_instrument.add_component(self.end_component_name, "Arm")
         end.set_AT([0, 0, self.get_length_name()], RELATIVE=slit)
 
-def plot_element_horizontal(ax, par_dict, color):
-    start = par_dict["start_point"]
-    start_width = par_dict["start_width"]
+    def get_geometry(self):
+        return GeometrySlit(start_point=self.start_point_parameter,
+                            next_start_point=self.next_start_point_parameter,
+                            start_width=self.start_width, start_height=self.start_height,
+                            end_width=self.end_width, end_height=self.end_height)
 
-    ax.plot([start, start], [0.5*start_width,  0.75*start_width], color=color)
-    ax.plot([start, start], [-0.5*start_width, -0.75*start_width], color=color)
 
-def plot_element_vertical(ax, par_dict, color):
-    start = par_dict["start_point"]
-    start_height = par_dict["start_height"]
+class GeometrySlit(BaseElementGeometry):
+    def __init__(self, start_point, next_start_point, start_width, start_height, end_width, end_height):
 
-    ax.plot([start, start], [0.5 * start_height, 0.75 * start_height], color=color)
-    ax.plot([start, start], [-0.5 * start_height, -0.75 * start_height], color=color)
+        super().__init__(start_point=start_point, next_start_point=next_start_point,
+                         start_width=start_width, start_height=start_height,
+                         end_width=end_width, end_height=end_height)
 
-def center_line(start_pr, par_dict, distance_unit_less):
+        self.visible = False
 
-    start = par_dict["start_point"]
-    end = par_dict["next_start_point"]
+    def plot_extra(self, ax, start_pr, horizontal, color):
+        """
+        Plot the outside of the slit to make it clear, not disabled by visible = False
+        """
+        inner_dim = self.get_dimension(0.0, horizontal)
+        if horizontal:
+            outer_dim = self.start_width * 1.5
+        else:
+            outer_dim = self.start_height * 1.5
 
-    length = end - start
-    this_position = length*distance_unit_less
+        inner_point_plus, inner_point_minus = start_pr.get_points(inner_dim, horizontal)
+        outer_point_plus, outer_point_minus = start_pr.get_points(outer_dim, horizontal)
 
-    z_vector = np.array([0, 0, 1])
-    start_direction = start_pr.rotation
-    z_vector = start_direction.apply(z_vector)
-
-    updated_position = start_pr.position + this_position*z_vector
-
-    return PositionAndRotation(updated_position, start_pr.rotation)
\ No newline at end of file
+        if horizontal:
+            ax.plot([inner_point_plus[2], outer_point_plus[2]],
+                    [inner_point_plus[0], outer_point_plus[0]], color=color)
+            ax.plot([inner_point_minus[2], outer_point_minus[2]],
+                    [inner_point_minus[0], outer_point_minus[0]], color=color)
+        else:
+            ax.plot([inner_point_plus[2], outer_point_plus[2]],
+                    [inner_point_plus[1], outer_point_plus[1]], color=color)
+            ax.plot([inner_point_minus[2], outer_point_minus[2]],
+                    [inner_point_minus[1], outer_point_minus[1]], color=color)
\ No newline at end of file
