@@ -114,8 +114,8 @@ class TestLengthSystem(unittest.TestCase):
         self.assertEqual(guide[0].start_point_parameter.get_value(), 0.0)
 
         self.assertIsInstance(guide[1].start_point_parameter, FreeInstrumentParameter)
-        self.assertEqual(guide[1].start_point_parameter.get_lower_static_bound(), 0.0)
-        self.assertEqual(guide[1].start_point_parameter.get_upper_static_bound(), 20.0)
+        self.assertEqual(guide[1].start_point_parameter.get_lower_static_bound(), 0.1)
+        self.assertEqual(guide[1].start_point_parameter.get_upper_static_bound(), 19.9)
 
         self.assertIsInstance(guide[1].next_start_point_parameter, FixedInstrumentParameter)
         self.assertEqual(guide[1].next_start_point_parameter.get_value(), 20.0)
@@ -173,12 +173,12 @@ class TestLengthSystem(unittest.TestCase):
         self.assertEqual(guide[0].start_point_parameter.get_value(), 0.0)
 
         self.assertIsInstance(guide[0].next_start_point_parameter, FreeInstrumentParameter)
-        self.assertEqual(guide[0].next_start_point_parameter.get_lower_static_bound(), 0.0)
-        self.assertEqual(guide[0].next_start_point_parameter.get_upper_static_bound(), 6.0)
+        self.assertEqual(guide[0].next_start_point_parameter.get_lower_static_bound(), 0.1)
+        self.assertEqual(guide[0].next_start_point_parameter.get_upper_static_bound(), 5.9)
 
         self.assertIsInstance(guide[1].start_point_parameter, FreeInstrumentParameter)
-        self.assertEqual(guide[1].start_point_parameter.get_lower_static_bound(), 0.0)
-        self.assertEqual(guide[1].start_point_parameter.get_upper_static_bound(), 6.0)
+        self.assertEqual(guide[1].start_point_parameter.get_lower_static_bound(), 0.1)
+        self.assertEqual(guide[1].start_point_parameter.get_upper_static_bound(), 5.9)
 
         self.assertIsInstance(guide[1].next_start_point_parameter, FixedInstrumentParameter)
         self.assertEqual(guide[1].next_start_point_parameter.get_value(), 6.0)
@@ -234,20 +234,20 @@ class TestLengthSystem(unittest.TestCase):
 
         self.assertIsInstance(guide[5].next_start_point_parameter, RelativeFreeInstrumentParameter)
         self.assertEqual(guide[5].next_start_point_parameter.static_lower, 11.0)
-        self.assertEqual(guide[5].next_start_point_parameter.static_upper, 30.0)
+        self.assertEqual(guide[5].next_start_point_parameter.static_upper, 29.9)
         lower_dynamic = guide[5].next_start_point_parameter.dynamic_lower[0]
         self.assertEqual(lower_dynamic.dependent_on[0], guide[5].start_point_parameter)
-        self.assertEqual(lower_dynamic.constants[0], 0.0)
+        self.assertEqual(lower_dynamic.constants[0], 0.1)
         self.assertEqual(lower_dynamic.function(10.0, 0.0), 10.0)
         self.assertEqual(len(guide[5].next_start_point_parameter.dynamic_lower), 1)
         self.assertEqual(len(guide[5].next_start_point_parameter.dynamic_upper), 0)
 
         self.assertIsInstance(guide[6].start_point_parameter, RelativeFreeInstrumentParameter)
         self.assertEqual(guide[6].start_point_parameter.static_lower, 11.0)
-        self.assertEqual(guide[6].start_point_parameter.static_upper, 30.0)
+        self.assertEqual(guide[6].start_point_parameter.static_upper, 29.9)
         lower_dynamic = guide[6].start_point_parameter.dynamic_lower[0]
         self.assertEqual(lower_dynamic.dependent_on[0], guide[5].start_point_parameter)
-        self.assertEqual(lower_dynamic.constants[0], 0.0)
+        self.assertEqual(lower_dynamic.constants[0], 0.1)
         self.assertEqual(lower_dynamic.function(10.0, 0.0), 10.0)
         self.assertEqual(len(guide[6].start_point_parameter.dynamic_lower), 1)
         self.assertEqual(len(guide[6].start_point_parameter.dynamic_upper), 0)
@@ -287,7 +287,7 @@ class TestLengthSystem(unittest.TestCase):
         total_length = 80.0
         length_system(guide, total_length, instr_parameters)
 
-        test_guide_configurations(self, guide, total_length)
+        run_test_guide_configurations(self, guide, total_length)
 
     def test_length_system_classic_ballistic_guide(self):
         guide = setup_classic_balistic_guide()
@@ -303,11 +303,11 @@ class TestLengthSystem(unittest.TestCase):
 
         self.assertIsInstance(guide[0].next_start_point_parameter, FreeInstrumentParameter)
         self.assertEqual(guide[0].next_start_point_parameter.get_lower_static_bound(), 2.0)
-        self.assertEqual(guide[0].next_start_point_parameter.get_upper_static_bound(), 6.5)
+        self.assertEqual(guide[0].next_start_point_parameter.get_upper_static_bound(), 6.4)
 
         self.assertIsInstance(guide[1].start_point_parameter, FreeInstrumentParameter)
         self.assertEqual(guide[1].start_point_parameter.get_lower_static_bound(), 2.0)
-        self.assertEqual(guide[1].start_point_parameter.get_upper_static_bound(), 6.5)
+        self.assertEqual(guide[1].start_point_parameter.get_upper_static_bound(), 6.4)
 
         self.assertIsInstance(guide[1].next_start_point_parameter, FixedInstrumentParameter)
         self.assertEqual(guide[1].next_start_point_parameter.get_value(), 6.5)
@@ -334,7 +334,7 @@ class TestLengthSystem(unittest.TestCase):
         self.assertEqual(guide[4].next_start_point_parameter.static_upper, 80.0 - 0.5 - 2.0)
         lower_dynamic = guide[4].next_start_point_parameter.dynamic_lower[0]
         self.assertEqual(lower_dynamic.dependent_on[0], guide[4].start_point_parameter)
-        self.assertEqual(lower_dynamic.constants[0], 0.0)
+        self.assertEqual(lower_dynamic.constants[0], 0.1)
         self.assertEqual(lower_dynamic.function(10.0, 1.0), 11.00)
         self.assertEqual(len(guide[4].next_start_point_parameter.dynamic_lower), 1)
         self.assertEqual(len(guide[4].next_start_point_parameter.dynamic_upper), 0)
@@ -344,7 +344,7 @@ class TestLengthSystem(unittest.TestCase):
         self.assertEqual(guide[5].start_point_parameter.static_upper, 80.0 - 0.5 - 2.0)
         lower_dynamic = guide[5].start_point_parameter.dynamic_lower[0]
         self.assertEqual(lower_dynamic.dependent_on[0], guide[4].start_point_parameter)
-        self.assertEqual(lower_dynamic.constants[0], 0.0)
+        self.assertEqual(lower_dynamic.constants[0], 0.1)
         self.assertEqual(lower_dynamic.function(10.0, 1.0), 11.00)
         self.assertEqual(len(guide[5].start_point_parameter.dynamic_lower), 1)
         self.assertEqual(len(guide[5].start_point_parameter.dynamic_upper), 0)
@@ -376,7 +376,7 @@ class TestLengthSystem(unittest.TestCase):
         total_length = 80.0
         length_system(guide, total_length, instr_parameters)
 
-        test_guide_configurations(self, guide, total_length, plot=False)
+        run_test_guide_configurations(self, guide, total_length, plot=False)
 
     def test_length_system_bug_guide_1(self):
         guide = setup_bug_example_1()
@@ -386,7 +386,7 @@ class TestLengthSystem(unittest.TestCase):
         print(instr_parameters)
 
 
-def test_guide_configurations(test_object, guide, total_length, plot=False):
+def run_test_guide_configurations(test_object, guide, total_length, plot=False):
     """
     Tests random configurations within allowed range of parameters
 
